

#include "U8glib.h"
#include "EEPROM.h"
/*
2센서 = 1
60바퀴 = 28cm
120센서 = 28cm
1센서 = 0.2333
1cm = 
*/

U8GLIB_SSD1306_128X32 u8g(U8G_I2C_OPT_NONE);	// I2C / TWI 

// drawBitmapP(X,Y,Count,H) 에서 count는 비트맵의 가로픽셀수/8, H는 세로픽셀수 이다. 
//최저높이 71cm 

//이하 핀 선언
#define encoderPin 2		//인터럽트 핀

#define DC_PWM_Pin 3  //PWM pin
#define DC_dir_Pin A1

#define UP_SW_Pin 12
#define Sel_SW_Pin 11
#define DN_SW_Pin 10

//이하 변수 선언


float current_Height = 70;							//현재높이(부팅시 시작하는 높이)
float toMove_Height = 0;							//이동할높이
float min_Height = 60, max_Height= 120;				//최저, 최고높이. 
unsigned long current_Millis, previous_Millis = 0;	//스위치가 눌린 시간을 체크할 millis시간 저장변수
unsigned long pressed_Time = 700;					//스위치가 눌린 시간을 기준할 변수. 이보다 길면 길게 눌린 것으로 한다. 단위 millisecond. 
unsigned long screenSaver_Time = 180000;	 		//3분=180000밀리초	//화면을 끌 시간을 기준할 변수. 입력 없이 이보다 긴 시간이 지나면 화면을 끈다. 단위 millisecond. 
char HeightChar[5];
int mode = 0;										//mode = 0 : 높이조정, 1 = 높이저장, 2 = 높이로드


int address = 0;									//EEPROM 주소 지정. 
int address_display;
char addressChar[1];

boolean sel_Flag;									//sel 스위치 플래그
boolean up_Flag, down_Flag;							//상승, 하강 플래그.
boolean save_Flag, load_Flag;						//저장, 로드 플래그.

const uint8_t PROGMEM save_Bitmap[] = {					//save icon
0x78, 0x84, 0x80, 0x78, 0x04, 0x84, 0x78
}; 

const uint8_t PROGMEM load_Bitmap[] = {					//load icon
0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xf8
}; 



void setup() {
	pinMode(encoderPin, INPUT);
	pinMode(UP_SW_Pin, INPUT);
	pinMode(Sel_SW_Pin, INPUT);
	pinMode(DN_SW_Pin, INPUT);

	//pinMode(DC_EN_Pin, OUTPUT);
	pinMode(DC_PWM_Pin, OUTPUT);
	pinMode(DC_dir_Pin, OUTPUT);

	attachInterrupt(digitalPinToInterrupt(encoderPin), encoder, RISING);	//인터럽트 선언, encoderPin에 falling신호가 들어오면 encoder함수 실행.

	address = 0;

	u8g.setFont(u8g_font_helvB18r);
	u8g.setFontRefHeightExtendedText();
	u8g.setDefaultForegroundColor();
	u8g.setFontPosTop();


	current_Millis = millis();
	Serial.begin(9600);

}


void loop() {

	switch (mode) {
		case 0:
			if(digitalRead(UP_SW_Pin) == LOW){				//UP스위치가 눌린 경우 up()함수호출
				up_Flag = true;
				up();
				current_Millis = millis();					//스위치가 눌린 시간 기록
				previous_Millis = current_Millis;
			}

			if(digitalRead(DN_SW_Pin) == LOW){				//DN스위치가 눌린 경우 down()함수호출
				down_Flag = true;
				down();
				current_Millis = millis();					//스위치가 눌린 시간 기록
				previous_Millis = current_Millis;
			}

			if((digitalRead(Sel_SW_Pin) == LOW)){			//sel스위치가 눌렸을 때 작동
				if(sel_Flag == false){						//최초 sel 스위치가 눌린 시간 기록
					previous_Millis = current_Millis;
				}
				current_Millis = millis();
				sel_Flag = true;
			}

			if((digitalRead(Sel_SW_Pin) == HIGH) && (sel_Flag == true)){	//sel 스위치에서 손을 떼었을 때 작동
				if((current_Millis - previous_Millis) >= pressed_Time){
					previous_Millis = current_Millis;
					mode = 1;												//저장 모드로
				}
				else {
					mode = 2;												//로드 모드로
				}

				sel_Flag = false;
			}

		break;

		case 1:															//저장 모드

			if((digitalRead(UP_SW_Pin) == LOW) && (up_Flag == false)){	//UP스위치가 눌린 경우 
				up_Flag = true;
				address = address + 5;												//address 증가
				if (address > 20){ address = 20;}							//범위제한
				current_Millis = millis();								//스위치가 눌린 시간 기록
			}

			if((digitalRead(DN_SW_Pin) == LOW) && (down_Flag == false)){//DN스위치가 눌린 경우
				down_Flag = true;
				address = address - 5;												//address 감소
				if (address < 0){ address = 0;}							//범위제한
				current_Millis = millis();								//스위치가 눌린 시간 기록
			}

			if(digitalRead(UP_SW_Pin) == HIGH){							//UP,DN 스위치를 떼면 플래그를 체크
				up_Flag = false;	
			}
			if(digitalRead(DN_SW_Pin) == HIGH){
				down_Flag = false;
			}
			//---------------------------------------------------------------
			if((digitalRead(Sel_SW_Pin) == LOW)){			//sel스위치가 눌렸을 때 작동
				if(sel_Flag == false){						//최초 sel 스위치가 눌린 시간 기록
					previous_Millis = current_Millis;
				}
				current_Millis = millis();
				sel_Flag = true;
			}

			if((digitalRead(Sel_SW_Pin) == HIGH) && (sel_Flag == true)){	//sel 스위치에서 손을 떼었을 때 작동
				if((current_Millis - previous_Millis) >= pressed_Time){
					previous_Millis = current_Millis;
					mode = 0;												//높이조정 모드로 변경
				}
				else {
					EEPROM.put(address, current_Height);
					Serial.print("Saved arress=");Serial.println(address);
					clear();
					delay(100);
					mode = 0;
				}

				sel_Flag = false;
			}

		break;

		case 2:
			if((digitalRead(UP_SW_Pin) == LOW) && (up_Flag == false)){	//UP스위치가 눌린 경우 
				up_Flag = true;
				address = address + 5;												//address 증가
				if (address > 20){ address = 20;}						//범위제한
				current_Millis = millis();								//스위치가 눌린 시간 기록
			}

			if((digitalRead(DN_SW_Pin) == LOW) && (down_Flag == false)){//DN스위치가 눌린 경우
				down_Flag = true;
				address = address - 5;												//address 감소
				if (address < 0){ address = 0;}							//범위제한
				current_Millis = millis();								//스위치가 눌린 시간 기록
			}

			if(digitalRead(UP_SW_Pin) == HIGH){							//UP,DN 스위치를 떼면 플래그를 체크
				up_Flag = false;	
			}
			if(digitalRead(DN_SW_Pin) == HIGH){
				down_Flag = false;
			}
			//---------------------------------------------------------------
			if((digitalRead(Sel_SW_Pin) == LOW)){			//sel스위치가 눌렸을 때 작동
				if(sel_Flag == false){						//최초 sel 스위치가 눌린 시간 기록
					previous_Millis = current_Millis;
				}
				current_Millis = millis();
				sel_Flag = true;
			}

			if((digitalRead(Sel_SW_Pin) == HIGH) && (sel_Flag == true)){	//sel 스위치에서 손을 떼었을 때 작동
				if((current_Millis - previous_Millis) >= pressed_Time){
					previous_Millis = current_Millis;
					mode = 0;												//높이조정 모드로 변경
				}
				else {
					EEPROM.get(address, toMove_Height);
					clear();
					delay(100);
					Serial.print("Address=");Serial.println(address); 
					Serial.print("current_Height=");Serial.println(current_Height);
					Serial.print("toMove_Height=");Serial.println(toMove_Height);

					while(current_Height < toMove_Height){
						up_Flag = true;
						draw();
						up();
						
					}
					while(current_Height > toMove_Height){
						down_Flag = true;
						draw();
						down();
						
					}
					if(current_Height == toMove_Height){
						toMove_Height = current_Height;
						mode = 0;
					}

				}
				sel_Flag = false;
			}

		break;
	}



	if((digitalRead(UP_SW_Pin)==HIGH)
		&&(digitalRead(DN_SW_Pin)==HIGH)&&(digitalRead(Sel_SW_Pin)==HIGH)){ //스위치가 눌러져 있지 않을때는 모터를 정지시킨다
		analogWrite(DC_PWM_Pin, 0);
		current_Millis = millis();
	}




	if((current_Millis - previous_Millis) > screenSaver_Time){		
		clear();				//스크린세이버
	}
	else{
		draw();
	}
}



void encoder(){	//엔코더핀에 인터럽트 신호가 들어오면 실행될 함수

	if(up_Flag==true){		//상승중
		current_Height += 0.23;
		up_Flag = false;
	}
	if(down_Flag==true){		//하강중
		current_Height -= 0.23;
		down_Flag = false;
	}
	Serial.println(current_Height);
}



void draw(){
    
    address_display = address;
	u8g.firstPage();  //화면에 기본 정보 표시 
	do {
		dtostrf(current_Height, 3, 0, HeightChar);		//높이 표시 
		if(current_Height>=99.5){				//세자리수 이상일땐 좌표를 이동해서 화면에 맞춤
			u8g.drawStr(5, 4, HeightChar);
		}
		else{
			u8g.drawStr(10, 4, HeightChar);
		}
		u8g.drawStr(45, 4, "cm");

		if(mode == 1){					//세이브 상황. 
		u8g.drawRBox(100, 1,24,29, 4);
			if(address > 0){
				dtostrf((address_display/5)+1, 1, 0, addressChar);	//저장된 위치 번호 표시. address에 1을 더하여 저장번호로 사용한다. 

			}
			else{
				dtostrf(address_display+1, 1, 0, addressChar);	//저장된 위치 번호 표시. address에 1을 더하여 저장번호로 사용한다. 
			}		
		u8g.setColorIndex(0);					//검은색으로 변경
		u8g.drawStr(106, 4, addressChar);
		u8g.setColorIndex(1);					//흰색으로 변경

		// drawBitmapP(X,Y,Count,H) 에서 count는 비트맵의 가로픽셀수/8, H는 세로픽셀수 이다. 
        u8g.drawBitmapP( 0, 0, 1, 7, save_Bitmap);	
		}	

		if(mode == 2){					//로드 상황. 
		u8g.drawRFrame(100, 1,24,29, 4);
			if(address > 0){
				dtostrf((address_display/5)+1, 1, 0, addressChar);	//저장된 위치 번호 표시. address에 1을 더하여 저장번호로 사용한다. 

			}
			else{
				dtostrf(address_display+1, 1, 0, addressChar);	//저장된 위치 번호 표시. address에 1을 더하여 저장번호로 사용한다. 
			}	
		u8g.drawStr(106, 4, addressChar);

        u8g.drawBitmapP( 0, 0, 1, 7, load_Bitmap);	
		}	


	}
	while(  u8g.nextPage() ) ;   
}


void clear(){
	u8g.firstPage(); 
		do {
		}
		while(  u8g.nextPage() );
}



void up(){
	if(current_Height < max_Height){	//최고 높이 이하일때만 상승
		analogWrite(DC_PWM_Pin, 255);
		digitalWrite(DC_dir_Pin,HIGH);
	}
	else{
		analogWrite(DC_PWM_Pin, 0);
	}
}

void down(){

	if(current_Height > min_Height){	//최저 높이 이상일때만 하강 
		analogWrite(DC_PWM_Pin, 200);
		digitalWrite(DC_dir_Pin,LOW);	
	}
	else{
		analogWrite(DC_PWM_Pin, 0);
	}
}